µPCN - A Free DTN Implementation for Microcontrollers
=====================================================

**Note:** Currently the documentation does not really deserve its name.
However, we are working on improving it over the course of the next releases.

Quick Start
-----------

Only the STM32F4 embedded system is supported currently.
However, porting to other Cortex-M3/M4 based SoCs should be trivial.

In order to run µPCN on bare metal (e.g. the STM32F4Discovery board)
three steps are necessary after connecting the board via STLink-enabled USB:

1. Install or unpack the following dependencies:
   - The `gcc-arm-none-eabi` toolchain including `newlib`.
   - STLinkV2, including the `st-flash` tool.
   - A recent version of FreeRTOS 7 or 8.
   - For debugging and testing you may need OpenOCD, GDB, GNU Expect,
     GCC for you local architecture and the ZeroMQ headers and library.

2. Copy `config.mk.example` to `config.mk` and set all variables according to
   your installation:
   - Set `TOOLCHAIN` to the prefix for your *arm-none-eabi* toolchain.
   - Set `FREERTOS_PATH` to the path to your FreeRTOS source.
   - Set `ST_FLASH` to the path to your `st-flash` tool.

3. Type `make burn` to build the project and to burn `upcn.bin` to the
   board attached via USB.

Getting Started with the Implementation
---------------------------------------

The core part of uPCN is located in `./components/upcn/`.
The starting point of the program can be found in
`./components/upcn/src/main.c`, calling init located in
`./components/upcn/src/init.c`.
This file is the best place to familiarize with the implementation.

Testing
-------

Tools for testing can be found in `./tools`.

To interface with the program and run the pre-defined test scenarios:

1. Type `make connect` to start the ZeroMQ-based server application
   `upcn_connect` for `/dev/ttyACM0`.

2. Type `make receive` in another terminal to view the output of µPCN
   on the screen.

3. Run a test or send a command using `upcn_test`:

        tools/build/upcn_test/upcn_test tcp://127.0.0.1:8726 \
            tcp://127.0.0.1:8727 <command>

For debugging the program, openocd can be used.

1. Start the `openocd` server from within the project's root directory.

2. Type `make gdb` to start GDB.

   **Note:** `openocd` also supports flashing the program.
   For that, type `make flash`. Make sure you have the `expect` tool installed.

To test the _RRND_ discovery component see the respective documentation
in `./tools/rrnd_test/README.md` and `./tools/rrnd_proxy/README.md`.

License
-------

The code in `./components/upcn`, `./components/test/{include,src}` and
`./tools/{upcn_connect,upcn_receive,upcn_test,rrnd_proxy,rrnd_test}`
has been developed specifically for µPCN and is released under a
BSD 3-clause license.
The license can be found in `./components/upcn/LICENSE.txt`.

External code
-------------

As an early starting point for the STM32F4 project structure,
we have used the project https://github.com/elliottt/stm32f4/
as a general basis.

All further code taken from 3rd parties is documented within the source files,
along with the respective original URLs and associated licenses.
Generally, 3rd party code is found in `./components/drv`.
