#include <FreeRTOS.h>
#include <task.h>

#include <unity_fixture.h>
#include <upcn.h>
#include <test.h>
#include <init.h>

#define WR(c) comm_write_raw(c, 1)

static int test_errors;

void upcntest_print(void)
{
	if (!test_errors)
		comm_write("uPCN unittests succeeded\n");
	else
		comm_printf("uPCN unittests resulted in %d error(s)\n",
			test_errors);
}

void test_task(void *args)
{
	static char *argv[1] = { "testupcn" };
	int8_t led, c;

	led_set(1);
	LOG("Blinking some LEDs...");
	for (led = 0; led < 5; led++) {
		led_set(led);
		vTaskDelay(100 / portTICK_PERIOD_MS);
	}
	for (led = 4; led >= 0; led--) {
		led_set(led);
		vTaskDelay(100 / portTICK_PERIOD_MS);
	}
	LOG("Starting testsuite...");
	comm_write("\n");
	/* Disable the logger spamming out output */
	upcn_dbg_log_disable_output();
	/* Start Unity */
	test_errors = UnityMain(1, argv, testupcn);
	upcn_dbg_log_enable_output();
	/* Blink to indicate that tests are done */
	led = 0;
	for (c = 0; c < 6; c++) {
		led_set(led = 1 - led);
		vTaskDelay(100 / portTICK_PERIOD_MS);
	}
	/* Print logfile */
	upcn_dbg_printlogs();
	/* Start upcn */
	LOG("Tests done, switching to uPCN...");
	start_tasks();
	/* Delete task */
	vTaskDelete(NULL);
}

int main(void)
{
	init();
	xTaskCreate(test_task, "test_task", 1024, NULL, 0, NULL);
	return start_os();
}
