#ifndef BUNDLEPARSER_H_INCLUDED
#define BUNDLEPARSER_H_INCLUDED

#include <stdint.h>

#include <upcn.h>
#include <parser.h>
#include <bundle.h>
#include <sdnv.h>

/**
 * Represents the part of the bundle the parser is currently processing
 * This serves to determine how incoming bytes are treated (and possibly stored)
 */
enum bundle_parser_stage {
	PARSER_STAGE_VERSION,
	PARSER_STAGE_PROC_FLAGS,
	PARSER_STAGE_BLOCK_LENGTH,
	PARSER_STAGE_DESTINATION_EID_SCHEME,
	PARSER_STAGE_DESTINATION_EID_SSP,
	PARSER_STAGE_SOURCE_EID_SCHEME,
	PARSER_STAGE_SOURCE_EID_SSP,
	PARSER_STAGE_REPORT_EID_SCHEME,
	PARSER_STAGE_REPORT_EID_SSP,
	PARSER_STAGE_CUSTODIAN_EID_SCHEME,
	PARSER_STAGE_CUSTODIAN_EID_SSP,
	PARSER_STAGE_TIMESTAMP,
	PARSER_STAGE_SEQUENCE_NUM,
	PARSER_STAGE_LIFETIME,
	PARSER_STAGE_DICT_LENGTH,
	PARSER_STAGE_DICTIONARY,
	PARSER_STAGE_FRAGMENT_OFFSET,
	PARSER_STAGE_ADU_LENGTH,
	PARSER_STAGE_BLOCK_TYPE,
	PARSER_STAGE_BLOCK_FLAGS,
	PARSER_STAGE_BLOCK_EID_REF_CNT,
	PARSER_STAGE_BLOCK_EID_REF_SCH,
	PARSER_STAGE_BLOCK_EID_REF_SSP,
	PARSER_STAGE_BLOCK_DATA_LENGTH,
	PARSER_STAGE_BLOCK_DATA,
	PARSER_STAGE_DONE
};

/**
 * Specifies the currently parsed block
 */
enum bundle_parser_block {
	PARSER_BLOCK_PRIMARY,
	PARSER_BLOCK_GENERIC,
	PARSER_BLOCK_PAYLOAD,
	PARSER_BLOCK_EXTENSION
};

/**
 * Detailed cause of an error, if one occured (see `enum parser_status`)
 */
enum bundle_parser_error {
	PARSER_ERROR_BLOCK_LENGTH_EXHAUSTED,
	PARSER_ERROR_SDNV_FAILURE,
	PARSER_ERROR_NONE
};

/**
 * Represents the parser state
 * if you add or change something here, please don't forget to update
 * parser_reset(), too!
 */
struct bundle_parser {
	struct parser *basedata;
	void (*send_callback)(struct bundle *);

	enum bundle_parser_error error;

	enum bundle_parser_stage current_stage;
	enum bundle_parser_stage next_stage;
	enum bundle_parser_block current_block;

	struct sdnv_state sdnv_state;
	struct bundle *bundle;

	uint16_t primary_bytes_remaining;
	uint32_t cur_bytes_remaining;
	uint32_t current_index;
	uint32_t current_size;

	struct bundle_block_list **current_block_entry;
	uint8_t last_block;
};

struct parser *bundle_parser_init(
	struct bundle_parser *state, void (*send_callback)(struct bundle *));
enum upcn_result bundle_parser_reset(struct bundle_parser *state);
void bundle_parser_read_byte(struct bundle_parser *state, uint8_t byte);

#endif /* BUNDLEPARSER_H_INCLUDED */
