#ifndef MPU_H_INCLUDED
#define MPU_H_INCLUDED

#ifdef INCLUDE_BOARD_LIB

#include <stm32f4xx.h>

/* Some arbitrary region numbers */
#define MPU_NULL_REGION (0x00UL << MPU_RNR_REGION_Pos)
#define MPU_CCSM_REGION (0x02UL << MPU_RNR_REGION_Pos)
#define MPU_SRAM_REGION (0x03UL << MPU_RNR_REGION_Pos)
#define MPU_FLSH_REGION (0x04UL << MPU_RNR_REGION_Pos)
#define MPU_PERI_REGION (0x05UL << MPU_RNR_REGION_Pos)
#define MPU_DWTR_REGION (0x06UL << MPU_RNR_REGION_Pos)

/* For regions, see prog. manual p. 27 (2.2 Memory model) */
/* For size values, see prog. manual p. 189f (4.2.9 MPU_RASR) => 2^(1+s) */
#define MPU_REGION_NULL_BASE (0x00000000UL) /* Flash is mapped at NULL */
#define MPU_REGION_NULL_SIZE MPU_REGION_FLSH_SIZE /* 1.0 MB */
#define MPU_REGION_CCSM_BASE CCMDATARAM_BASE
#define MPU_REGION_CCSM_SIZE (0x0FUL << MPU_RASR_SIZE_Pos) /* 64 KB */
#define MPU_REGION_SRAM_BASE SRAM_BASE
#define MPU_REGION_SRAM_SIZE (0x10UL << MPU_RASR_SIZE_Pos) /* 128 KB */
#define MPU_REGION_FLSH_BASE FLASH_BASE
#define MPU_REGION_FLSH_SIZE (0x13UL << MPU_RASR_SIZE_Pos) /* 1.0 MB */
#define MPU_REGION_PERI_BASE PERIPH_BASE
#define MPU_REGION_PERI_SIZE (0x1CUL << MPU_RASR_SIZE_Pos) /* 0.5 GB */
#define MPU_REGION_DWTR_BASE 0xE0001000UL
#define MPU_REGION_DWTR_SIZE (0x05UL << MPU_RASR_SIZE_Pos) /* 64 B */

/* RASR access flags */
#define MPU_ACCESS_NO (0x00UL << MPU_RASR_AP_Pos)
#define MPU_ACCESS_RW (0x03UL << MPU_RASR_AP_Pos)
#define MPU_ACCESS_RO (0x06UL << MPU_RASR_AP_Pos)
#define MPU_ACCESS_ROX (MPU_ACCESS_RO | MPU_RASR_XN_Msk)
#define MPU_ACCESS_RWX (MPU_ACCESS_RW | MPU_RASR_XN_Msk)
/* RASR enable flag */
#define MPU_REGION_ENABLE MPU_RASR_ENABLE_Msk

void mpu_init(void);

#endif /* INCLUDE_BOARD_LIB */

#endif /* MPU_H_INCLUDED */
