#include <upcn.h>
#include <mpu.h>

#ifdef INCLUDE_BOARD_LIB

#include <stm32f4xx.h>

void mpu_init(void)
{
	/* Disable MPU for configuration */
	MPU->CTRL &= ~MPU_CTRL_ENABLE_Msk;

	/* Region NULL: 1 MB @ 0x00000000 non-accessible */
	MPU->RNR  = MPU_NULL_REGION;
	MPU->RBAR = MPU_REGION_NULL_BASE;
	MPU->RASR = MPU_REGION_NULL_SIZE | MPU_ACCESS_NO | MPU_REGION_ENABLE;

	/* Region CCSM: CCSRAM 64 KB @ 0x10000000 RW */
	MPU->RNR  = MPU_CCSM_REGION;
	MPU->RBAR = MPU_REGION_CCSM_BASE;
	MPU->RASR = MPU_REGION_CCSM_SIZE | MPU_ACCESS_RW | MPU_REGION_ENABLE;

	/* Region SRAM: 128 KB @ 0x20000000 RW */
	MPU->RNR  = MPU_SRAM_REGION;
	MPU->RBAR = MPU_REGION_SRAM_BASE;
	MPU->RASR = MPU_REGION_SRAM_SIZE | MPU_ACCESS_RW | MPU_REGION_ENABLE;

	/* Region FLSH: FLASH 1 MB @ 0x08000000 RX */
	/* TODO: This crashess (SystemInit?) */
	/*MPU->RNR  = MPU_FLSH_REGION;
	 *MPU->RBAR = MPU_REGION_FLSH_BASE;
	 *MPU->RASR = MPU_REGION_FLSH_SIZE | MPU_ACCESS_ROX | MPU_REGION_ENABLE;
	 */

	/* Region PERI: PERIPHERAL 0.5 GB @ 0x40000000 RW */
	MPU->RNR  = MPU_PERI_REGION;
	MPU->RBAR = MPU_REGION_PERI_BASE;
	MPU->RASR = MPU_REGION_PERI_SIZE | MPU_ACCESS_RW | MPU_REGION_ENABLE;

#ifdef DEBUG
	/* Region DWTR: Data Watchpoint & Trace registers @ 0xE0001000 RW */
	MPU->RNR  = MPU_DWTR_REGION;
	MPU->RBAR = MPU_REGION_DWTR_BASE;
	MPU->RASR = MPU_REGION_DWTR_SIZE | MPU_ACCESS_RW | MPU_REGION_ENABLE;
#endif /* DEBUG */

	/* Enable MEMFAULT and MPU */
	SCB->SHCSR |= SCB_SHCSR_MEMFAULTENA_Msk;
	MPU->CTRL |= MPU_CTRL_ENABLE_Msk | MPU_CTRL_PRIVDEFENA_Msk;
}

#endif /* INCLUDE_BOARD_LIB */
