all: tools

tools: tools/build/upcn_connect/upcn_connect
tools: tools/build/upcn_receive/upcn_receive
tools: tools/build/upcn_test/upcn_test
tools: tools/build/rrnd_proxy/rrnd_proxy

LFLAGS_LOCAL := -lrt -lzmq -lm
CFLAGS_LOCAL += -std=gnu11

echo-cc-local = echo '  CC_L    $@';
cc-local = @$(call echo-cc-local,$@) gcc -o "$@" $(CFLAGS_LOCAL) -c "$<"

echo-link-local = echo '  LINK_L  $@';
link-local = @$(call echo-link-local,$@) gcc -o "$@" $(OBJECTS) $(LFLAGS_LOCAL)

echo-ar-local = echo '  AR_L    $@';
ar-local = @$(call echo-ar-local,$@) ar rcs "$@" $(OBJECTS)

# libupcn

libupcn_SOURCES := $(wildcard tools/libupcn/*.c)

libupcn_OBJECTS := $(patsubst tools/libupcn/%.c, \
	tools/build/libupcn/%.o, $(libupcn_SOURCES))

libupcn := tools/build/libupcn/libupcn.a

tools/build/libupcn/%.o: tools/libupcn/%.c
	$(call cc-local)

tools/build/libupcn/libupcn.a: OBJECTS := $(libupcn_OBJECTS)
tools/build/libupcn/libupcn.a: CFLAGS_LOCAL += -Itools/include/upcn
tools/build/libupcn/libupcn.a: $(libupcn_OBJECTS)
tools/build/libupcn/libupcn.a: | tools/build/libupcn
	$(call ar-local)

# upcn_connect

upcn_connect_SOURCES := $(wildcard tools/upcn_connect/*.c)

upcn_connect_OBJECTS := $(patsubst tools/upcn_connect/%.c, \
	tools/build/upcn_connect/%.o, $(upcn_connect_SOURCES))

tools/build/upcn_connect/%.o: tools/upcn_connect/%.c
	$(call cc-local)

tools/build/upcn_connect/upcn_connect: OBJECTS := $(upcn_connect_OBJECTS)
tools/build/upcn_connect/upcn_connect: CFLAGS_LOCAL += \
	-Itools/include -Itools/upcn_connect
tools/build/upcn_connect/upcn_connect: LFLAGS_LOCAL += -pthread
tools/build/upcn_connect/upcn_connect: $(upcn_connect_OBJECTS)
tools/build/upcn_connect/upcn_connect: | tools/build/upcn_connect
	$(call link-local)

# upcn_receive

upcn_receive_SOURCES := $(wildcard tools/upcn_receive/*.c)

upcn_receive_OBJECTS := $(patsubst tools/upcn_receive/%.c, \
	tools/build/upcn_receive/%.o, $(upcn_receive_SOURCES))

tools/build/upcn_receive/%.o: tools/upcn_receive/%.c
	$(call cc-local)

tools/build/upcn_receive/upcn_receive: OBJECTS := $(upcn_receive_OBJECTS)
tools/build/upcn_receive/upcn_receive: CFLAGS_LOCAL += \
	-Itools/include -Itools/upcn_receive
tools/build/upcn_receive/upcn_receive: $(upcn_receive_OBJECTS)
tools/build/upcn_receive/upcn_receive: | tools/build/upcn_receive
	$(call link-local)

# upcn_test

upcn_test_SOURCES := $(wildcard tools/upcn_test/*.c)

upcn_test_OBJECTS := $(patsubst tools/upcn_test/%.c, \
	tools/build/upcn_test/%.o, $(upcn_test_SOURCES))

tools/build/upcn_test/%.o: tools/upcn_test/%.c
	$(call cc-local)

tools/build/upcn_test/upcn_test: OBJECTS := $(upcn_test_OBJECTS) $(libupcn)
tools/build/upcn_test/upcn_test: CFLAGS_LOCAL += \
	-Itools/include -Itools/include/upcn -Itools/upcn_test
tools/build/upcn_test/upcn_test: $(upcn_test_OBJECTS)
tools/build/upcn_test/upcn_test: $(libupcn)
tools/build/upcn_test/upcn_test: | tools/build/upcn_test
	$(call link-local)

# rrnd_proxy

rrnd_proxy_SOURCES := $(wildcard tools/rrnd_proxy/*.c)

rrnd_proxy_OBJECTS := $(patsubst tools/rrnd_proxy/%.c, \
	tools/build/rrnd_proxy/%.o, $(rrnd_proxy_SOURCES))

tools/build/rrnd_proxy/%.o: tools/rrnd_proxy/%.c
	$(call cc-local)

tools/build/rrnd_proxy/rrnd_proxy: OBJECTS := \
	$(rrnd_proxy_OBJECTS) $(libupcn)
tools/build/rrnd_proxy/rrnd_proxy: CFLAGS_LOCAL += \
	-Itools/include -Itools/include/upcn -Itools/rrnd_proxy
tools/build/rrnd_proxy/rrnd_proxy: $(rrnd_proxy_OBJECTS)
tools/build/rrnd_proxy/rrnd_proxy: $(libupcn)
tools/build/rrnd_proxy/rrnd_proxy: | tools/build/rrnd_proxy
	$(call link-local)

# rrnd_perf_eval

tools/build/rrnd_perf_eval/%.o: tools/rrnd_perf_eval/%.c
	$(call cc-local)

tools/build/rrnd_perf_eval/%: CFLAGS_LOCAL += \
	-Itools/include -Itools/include/upcn -Itools/rrnd_perf_eval
tools/build/rrnd_perf_eval/%: $(libupcn)
tools/build/rrnd_perf_eval/%: | tools/build/rrnd_perf_eval
	$(call link-local)

# tools

tools/build/rrnd_perf_eval/sgp4_perf: OBJECTS := \
	tools/build/rrnd_perf_eval/sgp4_perf.o $(libupcn)
tools/build/rrnd_perf_eval/sgp4_perf: tools/build/rrnd_perf_eval/sgp4_perf.o
tools: tools/build/rrnd_perf_eval/sgp4_perf

tools/build/rrnd_perf_eval/mpfit_perf: OBJECTS := \
	tools/build/rrnd_perf_eval/mpfit_perf.o $(libupcn)
tools/build/rrnd_perf_eval/mpfit_perf: tools/build/rrnd_perf_eval/mpfit_perf.o
tools: tools/build/rrnd_perf_eval/mpfit_perf

tools/build/rrnd_perf_eval/ipnd_perf: OBJECTS := \
	tools/build/rrnd_perf_eval/ipnd_perf.o $(libupcn)
tools/build/rrnd_perf_eval/ipnd_perf: tools/build/rrnd_perf_eval/ipnd_perf.o
tools: tools/build/rrnd_perf_eval/ipnd_perf

# General

tools/build/libupcn: | tools/build
	$(call cmd,mkdir)

tools/build/upcn_connect: | tools/build
	$(call cmd,mkdir)

tools/build/upcn_receive: | tools/build
	$(call cmd,mkdir)

tools/build/upcn_test: | tools/build
	$(call cmd,mkdir)

tools/build/rrnd_proxy: | tools/build
	$(call cmd,mkdir)

tools/build/rrnd_perf_eval: | tools/build
	$(call cmd,mkdir)

tools/build:
	$(call cmd,mkdir)

clean::
	$(call cmd,rm,tools/build)

toolclean::
	$(call cmd,rm,tools/build)
