#include <stdint.h>

#include <upcn.h>
#include <bundle.h>
#include <sdnv.h>

#define write_bytes(bytes, data) \
	write(data, bytes)
#define serialize_u16(buffer, value) \
	write_bytes(sdnv_write_u16(buffer, value), buffer)
#define serialize_u32(buffer, value) \
	write_bytes(sdnv_write_u32(buffer, value), buffer)
#define serialize_u64(buffer, value) \
	write_bytes(sdnv_write_u64(buffer, value), buffer)

enum upcn_result bundle_serialize(
	struct bundle *bundle, void (*write)(const void *, const size_t))
{
	uint8_t buffer[MAX_SDNV_SIZE];
	struct bundle_block_list *cur_entry;
	struct eid_reference *cur_ref;
	uint16_t count;

	/* Write version field */
	write_bytes(1, &(bundle->protocol_version));
	/* Write SDNV values up to dictionary length */
	serialize_u32(buffer, bundle->proc_flags);
	serialize_u32(buffer, bundle->primary_block_length);
	serialize_u16(buffer, bundle->destination_eid.scheme_offset);
	serialize_u16(buffer, bundle->destination_eid.ssp_offset);
	serialize_u16(buffer, bundle->source_eid.scheme_offset);
	serialize_u16(buffer, bundle->source_eid.ssp_offset);
	serialize_u16(buffer, bundle->report_eid.scheme_offset);
	serialize_u16(buffer, bundle->report_eid.ssp_offset);
	serialize_u16(buffer, bundle->custodian_eid.scheme_offset);
	serialize_u16(buffer, bundle->custodian_eid.ssp_offset);
	serialize_u64(buffer, bundle->creation_timestamp);
	serialize_u16(buffer, bundle->sequence_number);
	serialize_u64(buffer, bundle->lifetime);
	serialize_u16(buffer, bundle->dict_length);
	/* Write dictionary byte array */
	write_bytes(bundle->dict_length, (uint8_t *)bundle->dict);
	/* Write remaining SDNV values */
	if (HAS_FLAG(bundle->proc_flags, BUNDLE_FLAG_IS_FRAGMENT)) {
		serialize_u32(buffer, bundle->fragment_offset);
		serialize_u32(buffer, bundle->total_adu_length);
	}
	/* Serialize bundle blocks */
	cur_entry = bundle->blocks;
	while (cur_entry != NULL) {
		write_bytes(1, &cur_entry->data->type);
		serialize_u32(buffer, cur_entry->data->flags);
		if (HAS_FLAG(cur_entry->data->flags,
			BUNDLE_BLOCK_FLAG_HAS_EID_REF_FIELD)
		) {
			count = cur_entry->data->eid_ref_count;
			serialize_u16(buffer, count);
			cur_ref = cur_entry->data->eid_refs;
			for (; count > 0; count--, cur_ref++) {
				serialize_u16(buffer, cur_ref->scheme_offset);
				serialize_u16(buffer, cur_ref->ssp_offset);
			}
		}
		serialize_u32(buffer, cur_entry->data->length);
		write_bytes(cur_entry->data->length, cur_entry->data->data);
		cur_entry = cur_entry->next;
	}
	/* TODO: Error checking */
	return UPCN_OK;
}
