#include <stdio.h>

#include <mini-printf.h>
#include <json_escape.h>
#include <rrnd.h>

static const struct rrnd_contact_info empty_contact_info;

size_t rrnd_print_gs_info(
	char *const buf, const size_t bufsize, const char *const eid,
	const struct rrnd_gs_info *const gs)
{
	char *out = buf;
	size_t bytes_written, eid_bytes;
	struct rrnd_contact_info p = empty_contact_info;

	if (bufsize < 15)
		goto fail;
	bytes_written = mini_snprintf(out, bufsize, "{\"eid\": \"");
	if (bytes_written >= bufsize)
		goto fail_fatal;
	out += bytes_written;
	eid_bytes = json_escape_string(out, bufsize - bytes_written, eid);
	bytes_written += eid_bytes;
	out += eid_bytes;
	if (bytes_written >= bufsize)
		goto fail;
	if (gs == NULL) {
		bytes_written += mini_snprintf(out, bufsize - bytes_written,
			"\", \"success\": false}");
		if (bytes_written >= bufsize)
			goto fail;
		return bytes_written;
	}
	if (gs->predictions.elements != 0)
		p = gs->predictions.last;
	/* TODO: Remove snprintf dependency! */
	bytes_written += snprintf(out, bufsize - bytes_written,
		"\", \"reliability\": %f, \"flags\": %u, " \
			"\"tx_bitrate\": %lu, \"rx_bitrate\": %lu, " \
			"\"last_contact_start_time\": %llu, " \
			"\"last_contact_end_time\": %llu, " \
			"\"beacon_period\": %hu, \"beacon_count\": %hu, " \
			"\"locator_elements\": %hhu, " \
			"\"location\": [%f, %f, %f], " \
			"\"ap_contacts\": %hhu, \"ap\": [%lu, %lu, %lu], " \
			"\"history_elements\": %hhu, " \
			"\"history_missed_contacts\": %d, " \
			"\"predictions\": %hhu, \"next_prediction\": " \
			"[%llu, %lu, %f, %f, %lu, %f], \"success\": true}",
		gs->reliability, gs->flags,
		gs->tx_bitrate, gs->rx_bitrate,
		gs->last_contact_start_time,
		gs->last_contact_end_time,
		gs->beacon_period, gs->beacon_count,
		gs->locator_data.elements,
		gs->location.x, gs->location.y, gs->location.z,
		gs->availability_pattern.contacts_honored,
		gs->availability_pattern.offset,
		gs->availability_pattern.on_time,
		gs->availability_pattern.period,
		gs->contact_history.elements,
		__builtin_popcount(gs->contact_history.failed_contacts_bitmap),
		gs->predictions.elements,
		p.start, p.duration, p.probability, p.max_elevation,
		p.capacity.mean, p.capacity.std_deviation
	);
	if (bytes_written <= bufsize)
		return bytes_written;
fail:
	bytes_written = mini_snprintf(buf, bufsize, "{\"success\": false}");
	if (bytes_written < bufsize)
		return bytes_written;
fail_fatal:
	if (bufsize > 3)
		return mini_snprintf(buf, bufsize, "{}");
	else
		return 0;
}
