#!/usr/bin/env python

import argparse
import json


def run():
    parser = argparse.ArgumentParser(description="Check test output file.")
    parser.add_argument("FILE", type=argparse.FileType("r"),
                        help="the file to be read")
    parser.add_argument("--failedgsmax", type=int, default=0,
                        help="the maximum for failed gs count")
    parser.add_argument("--failedbeaconmax", type=float, default=0.1,
                        help="the maximum failed beacon ratio, in percent")
    parser.add_argument("--failedpredictionmax", type=float, default=25.0,
                        help="the maximum failed prediction ratio, in percent")
    args = parser.parse_args()
    try:
        data = json.load(args.FILE)
    finally:
        args.FILE.close()
    # Assert GS verification failed less than specified
    print("Failed GS verifications: {}".format(data["verify_gs_failed"]))
    assert data["verify_gs_failed"] <= args.failedgsmax
    # Assert less beacons failed than the specified ratio
    assert data["failed_beacons"] <= data["beacons"]
    bfail = data["failed_beacons"] * 100.0 / data["beacons"]
    print("Failed beacon ratio: {} %".format(bfail))
    assert bfail <= args.failedbeaconmax
    # Assert maximum of failed predictions
    assert data["failed_predictions"] <= data["requested_predictions"]
    pfail = data["failed_predictions"] * 100.0 / data["requested_predictions"]
    print("Failed prediction ratio: {} %".format(pfail))
    assert pfail <= args.failedpredictionmax
    # Assert equal GS count over all runs
    gscount = [len(x.keys()) for x in data["run_data"]]
    assert gscount.count(gscount[0]) == len(gscount)
    print("Everything OK.")

if __name__ == "__main__":
    run()
